@@ -1,14 +1,14 @@
-# test_WFSClient_v1_0_0.R
+# test_WFSClient_v1_1_0.R
 # Author: Emmanuel Blondel <emmanuel.blondel1@gmail.com>
 #
-# Description: Integration tests for WFS Client version 1.0.0
+# Description: Integration tests for WFS Client version 1.1.0
 #=======================
 require(ows4R, quietly = TRUE)
 require(testthat)
 context("WFS")
 
-test_that("WFS 1.0.0",{
-  wfs <- WFSClient$new("http://localhost:8080/geoserver/wfs", "1.0.0", logger = "DEBUG")
+test_that("WFS 1.1.0",{
+  wfs <- WFSClient$new("http://localhost:8080/geoserver/wfs", "1.1.0", logger = "INFO")
   expect_is(wfs, "WFSClient")
   caps <- wfs$getCapabilities()
   expect_is(caps, "WFSCapabilities")
